@@ -31,11 +31,26 @@
   unlist(lapply(cs,function(x) .jcall(x,"S","toString")))
 }
 
+### this list maps R class names to Java class names for which the constructor does the necessary conversion (for use in .jrcall)
+.class.to.jclass <-    c(character= "java/lang/String",
+                         jbyte    = "java/lang/Byte",
+                         integer  = "java/lang/Integer",
+                         numeric  = "java/lang/Double",
+                         logical  = "java/lang/Boolean",
+                         jlong    = "java/lang/Long",
+                         jchar    = "java/lang/Character",
+                         jshort   = "java/lang/Short",
+                         jfloat   = "java/lang/Float")
+
+### Java classes that have a corresponding primitive type and thus a corresponding TYPE field to use with scalars
+.primitive.classes = c("java/lang/Byte", "java/lang/Integer", "java/lang/Double", "java/lang/Boolean",
+                       "java/lang/Long", "java/lang/Character", "java/lang/Short", "java/lang/Float")
+
 ### reflected call - this high-level call uses reflection to call a method
 ### it is much less efficient than .jcall but doesn't require return type
 ### specification or exact matching of parameter types
 .jrcall <- function(o, method, ..., simplify=TRUE) {
-  if (!is.character(method) | length(method)!=1)
+  if (!is.character(method) | length(method) != 1)
     stop("Invalid method name - must be exactly one character string.")
   if (inherits(o, "jobjRef") || inherits(o, "jarrayRef"))
     cl <- .jcall(o, "Ljava/lang/Class;", "getClass")
@@ -43,40 +58,30 @@
     cl <- .jfindClass(o)
   if (is.null(cl))
     stop("Cannot find class of the object.")
-  p <- list(...)
-  ar <- .jcall("java/lang/reflect/Array", "Ljava/lang/Object;", "newInstance", .jclassClass, as.integer(length(p)))
-  op <- .jcall("java/lang/reflect/Array", "Ljava/lang/Object;", "newInstance", .jclassObject, as.integer(length(p)))
-  if (length(p)>0) for (i in 1:length(p)) {
-    if (inherits(p[[i]], "jobjRef") || inherits(o, "jarrayRef")) {
-      .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jcall(p[[i]],"Ljava/lang/Class;","getClass"),"java/lang/Object"))
-      .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(p[[i]],"java/lang/Object"))
-    } else {
-      pc <- class(p[[i]])
-      if (pc == "character" & length(p[[i]]) == 1) {
-        .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jclassString, "java/lang/Object"))
-        .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(.jnew("java/lang/String",p[[i]]), "java/lang/Object"))
-      } else if (pc == "integer" & length(p[[i]]) == 1) {
-        .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jclass.int, "java/lang/Object"))
-        .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(.jnew("java/lang/Integer",p[[i]]), "java/lang/Object"))
-      } else if (pc == "jfloat" & length(p[[i]]) == 1) {
-        .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jclass.float, "java/lang/Object"))
-        .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(.jnew("java/lang/Float",p[[i]]), "java/lang/Object"))
-      } else if (pc == "numeric" & length(p[[i]]) == 1) {
-        .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jclass.double, "java/lang/Object"))
-        .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(.jnew("java/lang/Double",p[[i]]), "java/lang/Object"))
-      } else if (pc == "logical" & length(p[[i]]) == 1) {
-        .jcall("java/lang/reflect/Array","V","set",.jcast(ar,"java/lang/Object"), as.integer(i-1),.jcast(.jclass.boolean, "java/lang/Object"))
-        .jcall("java/lang/reflect/Array","V","set",.jcast(op,"java/lang/Object"), as.integer(i-1),.jcast(.jnew("java/lang/Boolean",p[[i]]), "java/lang/Object"))
-      } else
-      stop("Sorry, parameter type `",pc,"' is not supported.")
+  
+  # p is a list of parameters that are formed solely by valid Java objects
+  p <- lapply(list(...), function(a) {
+    if (inherits(a, "jobjRef") || inherits(a, "jarrayRef")) a 
+    else if (is.null(a)) .jnull() else {
+      cm <- match(class(a)[1], names(.class.to.jclass))
+      if (!any(is.na(cm))) { if (length(a) == 1) { y <- .jnew(.class.to.jclass[cm], a); if (.class.to.jclass[cm] %in% .primitive.classes) attr(y, "primitive") <- TRUE; y } else .jarray(a) }
+      else
+        stop("Sorry, parameter type `",pc,"' is ambiguous or not supported.")
     }
-  }
-  m<-.jcall(cl, "Ljava/lang/reflect/Method;", "getMethod", method, .jcast(ar,"[Ljava/lang/Class;"))
+  })
+
+  # pc is a list of class objects
+  pc <- lapply(p, function(x) if (isTRUE(attr(x, "primitive"))) .jfield(x, "Ljava/lang/Class;", "TYPE") else .jcall(x, "Ljava/lang/Class;", "getClass"))
+
+  # use RJavaTools.getMethod instead of reflection since we need to match parameters (thanks to Romain Francois for the idea and code)
+  m <- .jcall("RJavaTools", "Ljava/lang/reflect/Method;", "getMethod", cl, method, .jarray(pc,"java/lang/Class"))
   if (is.null(m))
     stop("Cannot find Java method `",method,"' matching the supplied parameters.")
-  r<-.jcall(m, "Ljava/lang/Object;", "invoke", .jcast(if(inherits(o,"jobjRef") || inherits(o, "jarrayRef")) o else cl, "java/lang/Object"), .jcast(op, "[Ljava/lang/Object;"))
-  if (simplify && !is.jnull(r)) r <- .jsimplify(r)
-  r
+  ret <- .jcall(m, "Ljava/lang/Class;", "getReturnType")
+  r <- .jcall(m, "Ljava/lang/Object;", "invoke", .jcast(if(inherits(o,"jobjRef") || inherits(o, "jarrayRef")) o else cl, "java/lang/Object"), .jarray(p, "java/lang/Object"))
+  if (simplify && !is.jnull(r)) .jsimplify(r) else
+  if (is.jnull(r) && .jcall(m, "Ljava/lang/Class;", "getReturnType") == .jclass.void) invisible(NULL)
+  else r
 }
 
 ### simplify non-scalar reference to a scalar object if possible
@@ -85,7 +90,7 @@
     return(o)
   cn <- .jclass(o, true=TRUE)
   if (cn == "java.lang.Boolean") .jcall(o, "Z", "booleanValue") else
-  if (cn == "java.lang.Integer" || cn == "java.lang.Short" || cn == "java.lang.Charabter" || cn == "java.lang.Byte") .jcall(o, "I", "intValue") else
+  if (cn == "java.lang.Integer" || cn == "java.lang.Short" || cn == "java.lang.Character" || cn == "java.lang.Byte") .jcall(o, "I", "intValue") else
   if (cn == "java.lang.Number" || cn == "java.lang.Double" || cn == "java.lang.Long" || cn == "java.lang.Float") .jcall(o, "D", "doubleValue") else
   if (cn == "java.lang.String") .jstrVal(.jcast(o, "java/lang/String")) else
   o
@@ -136,7 +141,7 @@ setMethod("$", c(x="jobjRef"), function(x, name) {
   if (is.null(f))
     function(...) .jrcall(x, name, ...)
   else
-    .jfield(x, name)
+    .jfield(x, , name)
 })
 
 ### support for object$field<-...
@@ -147,3 +152,18 @@ setMethod("$<-", c(x="jobjRef"), function(x, name, value) .jfield(x, name) <- va
   if (true) .jcall(.jcall(o, "Ljava/lang/Class;", "getClass"), "S", "getName")
   else o@jclass
 }
+
+### support for names (useful for completion, thanks to Romain Francois)
+setMethod("names", c(x="jobjRef"), function(x) {
+  cl <- .jcall(x, "Ljava/lang/Class;", "getClass")
+
+  fields <- .jcall(cl, "[Ljava/lang/reflect/Field;", "getFields")
+  fieldnames <- sapply(fields, function(f) .jcall( f, "Ljava/lang/String;", "getName"))
+
+  methodz <- .jcall(cl, "[Ljava/lang/reflect/Method;", "getMethods")
+  methodnames <- sapply(methodz, function(m) .jcall( m, "Ljava/lang/String;", "getName"))
+
+  nargs  <- sapply(methodz, function(m) length(.jcall(m, "[Ljava/lang/Class;", "getParameterTypes" )))
+  methodnames <- paste(methodnames, ifelse( nargs == 0 , "()", "(" ), sep = "")
+  c(fieldnames, methodnames)
+})
