@@ -34,7 +34,12 @@ static JavaVMInitArgs vm_args;
 #define H_OUT  1
 #define H_EXIT 2
 
+#ifdef Win32
+/* the hooks are reportedly causing problems on Windows, so disable them by default */
+static int default_hooks = 0;
+#else
 static int default_hooks = H_OUT|H_EXIT;
+#endif
 
 static int JNICALL vfprintf_hook(FILE *f, const char *fmt, va_list ap) {
 #ifdef Win32
@@ -136,6 +141,8 @@ pthread_mutex_t initMutex = PTHREAD_MUTEX_INITIALIZER;
 int thInitResult = 0;
 int initAWT = 0;
 
+static void init_rJava(void);
+
 static void *initJVMthread(void *classpath)
 {
   int ws;
